# frozen_string_literal: true

# `Wouter::Route` represents a route definition. Example:
#
#     get '/users/:id/comments/:comment_id', RouteKlass
#
# It contains utility methods for matching a Rack request to a route.
#
class Wouter::Route
  REGEX = '([A-Za-z0-9\-\_]+)'

  attr_reader :request_method, :path, :klass

  def initialize(request_method:, path:, klass:)
    @request_method = request_method
    @path = path
    @klass = klass
  end

  # Expects `request` to be a `Rack::Request`
  # Check `request.request_method` and `request.path`
  def match?(request)
    return false if request.request_method.to_sym != @request_method
    return true if request.path == path

    path_regex =~ request.path
  end

  # Expects `request` to be a `Rack::Request`
  # Parse the path parameters out of `request.path`
  def params(request)
    request.path.match(named_path_regex)&.named_captures
  end

  private

  # /user/:id/comments/:comment_id
  # /\/user\/(?<id>\w+)\/comments\/(?<comment_id>\w+)/
  def named_path_regex
    @named_path_regex ||= begin
      string_regex = path
      path.scan(/(:\w+)/).flatten.each do |param|
        string_regex = string_regex.gsub(param, '(?<' + param[1..-1] +'>' + REGEX + ')')
      end
      Regexp.new('^' + string_regex + '$')
    end
  end

  # /user/:id/comments/:comment_id
  # /\/user\/(\w+)\/comments\/(\w+)/
  def path_regex
    @path_regex ||= begin
      string_regex = path
      path.scan(/(:\w+)/).flatten.each do |param|
        string_regex = string_regex.gsub(param, REGEX)
      end
      Regexp.new('^' + string_regex + '$')
    end
  end
end
